/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    canbus_master.c

    Routines to simulate a simple CANBUS master.

    2009-07-31	CF Lin
		new release
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mxcanbus.h"

#define MAX_MSG_SIZE			30

static unsigned int g_count = 0;

static void
print_can_message(CANMSG *msg)
{
	unsigned int i;

	printf("[0X%X 0X%X %d] ",msg->identifier,msg->flags, msg->length);
	for (i=0; i < msg->length; i++) printf("0X%X ", msg->data[i]);
	printf("\n");
}

void 
read_routine(unsigned int fd)
{
	CANMSG *msg;
	int max, packets=0;
	char buf[2048];
	unsigned int n,l;

	while(1)
	{
		max = mxcan_read(fd, buf, 2048, NULL);
		if (max < 0)
			break;
		msg = (CANMSG*) buf;
		while(max>=sizeof(CANMSG))
		{
			n = (unsigned int) atoi(msg->data);
			l = (unsigned int) (strlen(msg->data)+1);
			if (msg->length!=l || n!=g_count)
			{
				printf("read error = length %d vs %d n %d vs %d\n", msg->length, l, n, g_count);
				g_count = n; /* might be overrun, change the seq. */
			}
			g_count = (g_count+1)%1000000;
			msg++;
			packets++;
			max-=sizeof(CANMSG);
		}
		printf("read packets = %d\n", packets);
	}
}

void 
write_routine(unsigned int fd)
{
	CANMSG *msg;
	char buf[2048];
	int i, nw, len, packets=0;
	int ns = MAX_MSG_SIZE*sizeof(CANMSG);


	/* for testing, should perge old buffer */
	mxcan_purge_buffer(fd, 2);

	while(1)
	{
		msg = (CANMSG*) buf;
		/* send 30 canmsg */
		for (i=0;i < MAX_MSG_SIZE; i++, msg++)
		{
			msg->identifier = 0x27F;
			msg->flags = EXTENDED_FRAME;
			sprintf(msg->data, "%d", g_count);
			msg->length = strlen(msg->data)+1;
			g_count = (g_count+1)%1000000;
		}

		/* make sure all data is write */
		nw = 0;
		while(1) {
			len= mxcan_write(fd, buf+nw, ns-nw, NULL);
			if (len < 0) {
				printf("mxcan_write: error len=%d ns=%d nw=%d\n", len, ns, nw);
				break;
			} 
			nw +=len;
			if (nw == ns) 
				break;
			else 
				printf("write again nw=%d \n", nw);
		}
		packets += nw/sizeof(CANMSG);
		printf("write packets = %d\n", packets);

	}
}

static int
test_get_stat(unsigned int fd)
{
	int rtn;
	CANBST st;

	printf("++test_get_stat\n");
	rtn = mxcan_get_stat(fd, &st);
	if (rtn==0)
	{
		printf("data_overrun_count  %u\n",	st.data_overrun_count);
		printf("warning_error_count %u\n",	st.warning_error_count);
		printf("bus_error_count     %u\n",	st.bus_error_count);
		printf("status_error_count  %u\n",	st.status_error_count);
	}
	printf("--test_get_stat\n");
	return rtn;
}

static int
test_get_registers(unsigned int fd)
{
	int i, rtn;
	unsigned char buffer[256];

	printf("++test_get_registers\n");
	rtn = mxcan_get_registers(fd, buffer, 32);
	if (rtn)
		return -1;
	for (i=0;i < 32;i++)
	{
		printf("0X%02X ", buffer[i]);
		if (i%8==7)	printf("\n");
	}
	printf("--test_get_registers\n");
	return 0;
}

int
test_set_nonblocking(unsigned int fd)
{
	int rtn;

	printf("++test_set_nonblocking\n");
	rtn = mxcan_set_nonblocking(fd);
	printf("--test_set_nonblocking\n");

	return rtn;
}

/* the list of bus timing items that are supported */
static unsigned int bus_timing_tab[] = 
{	5, 10, 20, 40, 50, 80, 100, 125, 200, 250, 400, 500, 666, 800, 1000	};

int
test_bus_timing(unsigned int fd)
{
	int i, rtn, max = sizeof(bus_timing_tab)/sizeof(bus_timing_tab[0]);
	unsigned int speed;

	printf("++test_bus_timing\n");
	for (i=0;i < max; i++)
	{
		rtn = mxcan_set_bus_timing(fd, bus_timing_tab[i]);
		if (rtn)
			return -1;
		speed = mxcan_get_bus_timing(fd);
		if (speed!=bus_timing_tab[i])
			return -2;
	}
	printf("--test_bus_timing\n");
	return 0;
}

int
test_purge_buffer(unsigned int fd)
{
	int ret;

	printf("++test_purge_buffer\n");
	ret = mxcan_purge_buffer(fd, 1);
	if (ret != 0)
	{
		printf("mxcan_purge_buffer(fd,1): %d\n", ret);
		return -1;
	}
	ret = mxcan_inqueue(fd);
	if (ret != 0)
	{
		printf("mxcan_inqueue(fd): %d\n", ret);
		return -2;
	}
	ret = mxcan_purge_buffer(fd, 2);
	if (ret != 0)
	{
		printf("mxcan_purge_buffer(fd,2): %d\n", ret);
		return -3;
	}
	ret = mxcan_outqueue(fd);
	if (ret != 0)
	{
		printf("mxcan_outqueue(fd): %d\n", ret);
		return -4;
	}
	ret = mxcan_purge_buffer(fd, 3);
	if (ret != 0)
	{
		printf("mxcan_purge_buffer(fd,3): %d\n", ret);
		return -5;
	}
	if (mxcan_outqueue(fd) != 0 || mxcan_inqueue(fd) != 0)
	{
		printf("mxcan_outqueue and mxcan_inqueue\n");
		return -6;
	}
	printf("--test_purge_buffer\n");
	return 0;
}

void usage(void)
{
	printf("progname -p <port#> [-s <write sleep in ms>] [-w (write)] [-h (help)] [-f (check api)]\n");
}

#if defined(_WIN32_WCE) || defined(WIN32)

#ifndef EOF
#define EOF -1
#endif

int optind = 1;
char *optarg;

/* -b 500 -v */
int
getopt (int ac, char **av, const char *opts)
{
    const char *p;

    while (optind < ac)
    {
	if (*av[optind] == '-')
	{
	    p = strchr (opts, av[optind][1]);
	    if (!p)
		return '?';
	    if (*(p + 1) == ':')
	    {
		/* requires arg */
		optind++;
		if (!av[optind] || *av[optind] == '-')
		    return ':';	/* missing argument */
		optarg = av[optind];
	    }
	    else
		optarg = 0;
	    optind++;
	    return *p;
	}
	else
	    break;
    }
    optarg = 0;
    return EOF;
}

/* returns: number of fields found. */
int
split_line (char **temp, int tempcount, char *pkt)
{
    int     i = 0;

    if (!pkt)
	return -1;
    while (isspace (*pkt))
	pkt++;
    while (*pkt && i < tempcount)
    {
	if (*pkt == '"')
	{
	    /* quoted string */
	    pkt++;
	    temp[i++] = pkt;
	    pkt = strchr (pkt, '"');
	    if (!pkt)
	    {
		/* bogus line */
		return -1;
	    }
	    *pkt++ = 0;
	    if (!*pkt)
		break;
	    pkt++;		/* skip the space */
	}
	else
	{
	    temp[i++] = pkt;
	    pkt = strpbrk (pkt, " \t\r\n");
	    if (!pkt)
		break;
	    *pkt++ = 0;
	}
	while (isspace (*pkt))
	    pkt++;
    }
    return i;
}
#else
#include <getopt.h>
#endif

#if defined(_WIN32_WCE) || defined(WIN32)
int WINAPI 
WinMain( HINSTANCE hInstance,HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow )
#else
int main(int argc, char *argv[])
#endif
{
	unsigned int fd;
	int i, port = 0, ms = 0, check_api = 0, writer=0;

#if defined(_WIN32_WCE)
	int		argc;
	char	cmdline[256], *argv[32];

	WideCharToMultiByte(CP_ACP, 0, (LPCTSTR)lpCmdLine, 255, cmdline, 256, NULL, NULL);
	argc = split_line(argv+1, 32, cmdline)+1;
#endif /* !WIN32 */

	while ((i = getopt (argc, argv, "p:s:wfh")) != -1)
    {
		switch (i)
		{
		case 'p': /* port */
			port = atoi(optarg);
			break;
		case 's': /* sleep in ms */
			ms = atoi(optarg);
			break;
		case 'w': /* sleep in ms */
			writer = 1;
			break;
		case 'f': /* check function API */
			check_api = 1;
			break;
		case 'h': /* usage */
			usage();
			break;
		}
	}
	if (port==0)
	{
		usage();
		return 1;
	}
	fd = mxcan_open(port);
	if (fd ==0 )
	{
		printf("fail to open CANBUS port %d\n", port);
		return -1;
	}
	if (check_api)
	{
		if (test_get_stat(fd))
			return -1;
		if (test_get_registers(fd))
			return -2;
		if (test_set_nonblocking(fd))
			return -3;
		if (test_purge_buffer(fd))
			return -4;
		if (test_bus_timing(fd))
			return -5;
	}
	else if (writer)
		write_routine(fd);
	else
		read_routine(fd);
	mxcan_close(fd);
	return 0;
}

