/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/*
    mxdgio.c

    Routines to access digital I/O.

    2008-07-15	CF Lin
		new release
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#ifdef UC74XX
#define MXDIO_DEV	"/dev/lcm"
#elif (defined EPM3438)  // Jared, 08-10-2010, for EPM-3438
#define MXDIO_DEV	"/dev/epm_3438_dio1"
#define MXDIO_DEV2	"/dev/epm_3438_dio2"
#define COUNTER_NODE1	"/dev/epm_3438_counter1" // The counter of first EPM-3438 
#define COUNTER_NODE2	"/dev/epm_3438_counter2" // The counter of second EPM-3438
#else
#define MXDIO_DEV	"/dev/dio"
#endif

#define IOCTL_SET_DOUT	15
#define IOCTL_GET_DOUT	16
#define IOCTL_GET_DIN	17
#define IOCTL_RESET_COUNTER	18	// IO control for reseting the counter

typedef struct _DGTIO 
{
    int	port;
    int	data;
} DGTIO;

/*	open an access to digit io pins
		<brdno> the first or second board.
		0 for the first EPM-3438 counter. 1 for the second EPM-3438 counter.
	Returns:
		a file descriptor of the access; -1 for open fail
*/
#ifdef EPM3438  // Jared, 08-10-2010, for EPM-3438
int
mxdgio_epm3438_open(int brdno)
{
    if ( brdno == 0 )
        return open(MXDIO_DEV, O_RDWR);
    else
        return open(MXDIO_DEV2, O_RDWR);

    return -1;
}

/*
 * get the counter value
	Inputs:
		<fd> access to the conter device
	Returns:
		the counter value
 */
int mxdgio_epm3438_get_counter(int fd) {
	int len;
	unsigned int counter_value;

	len=read(fd, &counter_value , sizeof(unsigned int));
	if ( len < 0 )
		return -1;
	return counter_value;
}

/*
 * Clear the counter value
	Inputs:
		<fd> access to the conter device
	Returns:
		0:clear success; -n: clear fail
 */
int mxdgio_epm3438_clear_counter(int fd) {

	return ioctl(fd, IOCTL_RESET_COUNTER, NULL);;
}

#endif // Jared, 08-10-2010, end of #ifdef EPM-3438

int
mxdgio_open(void)
{
    return open(MXDIO_DEV, O_RDWR);
}

/*  get the signal state of a digit input 
    Inputs:
	<fd> the access to the device
	<port> port #
    Returns:
	1 or 0 on success, -1 on failure
*/
int
mxdgio_get_input_signal(int fd, int port)
{
    int n;
    DGTIO dio;

    dio.port = port;
    n = ioctl(fd, IOCTL_GET_DIN, &dio);
    return ((n>0)? -1:((dio.data>0)? 1:0));
}

/*	get the signal state of a digit output 
	Inputs:
		<fd> the access to the device
		<port> port #
	Returns:
		1 or 0 on success, -1 on failure
*/
int	
mxdgio_get_output_signal(int fd, int port)
{
    int n;
    DGTIO dio;

    dio.port = port;
    n = ioctl(fd, IOCTL_GET_DOUT, &dio);
    return ((n>0)? -1:((dio.data>0)? 1:0));
}

/*  pass a high signal to a digit output 
    Inputs:
	<fd> the access to the device
	<port> port #
    Returns:
	0 on success, -1 on failure
*/
int	
mxdgio_set_output_signal_high(int fd, int port)
{
    DGTIO dio;

    dio.port = port;
    dio.data = 1;
    return ioctl(fd, IOCTL_SET_DOUT, &dio);
}

/*  pass a low signal to a digit output 
    Inputs:
	<fd> the access to the device
	<port> port #
    Returns:
	0 on success, -1 on failure
*/
int	
mxdgio_set_output_signal_low(int fd, int port)
{
    DGTIO dio;

    dio.port = port;
    dio.data = 0;
    return ioctl(fd, IOCTL_SET_DOUT, &dio);
}

void
mxdgio_close(int fd)
{
    close(fd);
}

